/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const { Op } = require("sequelize")
const phoneUtil = require('google-libphonenumber').PhoneNumberUtil.getInstance();
const notifier = require("machankura.lib/notifier")

const supportedRedirects = [
    new RegExp("^/receive/$"),
    new RegExp("^/send/$")
]

module.exports = function () {
    var router = express.Router();

    router.use((request, response, next) => {

        console.log("Phone Number Paths: ", request.path)
        next()
    });

    router.route('/')
        .post(async function(request, response, next)  {
            const countries = await db.Country.findAll({
                include: [
                    {
                        association: db.Country.ServiceCountry,
                        required: true
                    }
                ]
            })
            
            try {
                const countryCallingCode = phoneUtil.parse(request.body.phoneNumber).getCountryCode()
                const country = countries.find(c => c.countryCallingCode == countryCallingCode)
                
                if (country) {
                    const normalizedReceiver = notifier.normalizeReceiverNumberByCountry(
                        country,
                        request.body.phoneNumber
                    ).replace("+", "").replaceAll("-", "").replaceAll(" ", "")
    
                    // find user 
                    const user = await db.User.findOne({
                        include: [
                            {
                                association: db.User.UserPhones,
                                include: [
                                    {
                                        association: db.UserPhone.Phone,
                                        where: {
                                            number: normalizedReceiver
                                        }
                                    }
                                ]
                            }
                        ]
                    }).catch(error => {
                        next(error)
                        return null
                    })
    
                    if (user) {
                        // Destroy all other OTPs
                        await db.OneTimePIN.destroy({
                            where: {
                                phoneNumber: normalizedReceiver
                            },
                            force: true
                        })

                        // Find or Create a Device with deviceId
                        

                        // Generate OTP
                        const oneTimePIN = await db.OneTimePIN.create({
                            phoneNumber: normalizedReceiver,
                            otpDeviceId: request.body.deviceId // TODO: Should be otpUserDeviceId
                        }).catch(error => {
                            console.error("Error: ", error)
                            next(error)
                            return null
                        })

                        if (oneTimePIN) {
                            // Send OTP SMS...
                            const result = await notifier.notifyUserOfTemporaryOTP(
                                db,
                                normalizedReceiver,
                                oneTimePIN.otp
                            )

                            if (result) {
                                // Show OTP Page...
                                response.json({
                                    // status: "ok",
                                    message: "Sent OTP."
                                })
                                db.OneTimePINSMS.create({
                                    phoneNumber: normalizedReceiver
                                }).then(otp => {
                                    
                                }).catch(error => {
                                    console.log("Error creating OTP SMS: ", error)
                                })
                            } else {
                                // May have failed to send sms...
                                response.status(500)
                                response.json({
                                    message: "failed to send SMS"
                                })
                            }                            
                        } else {
                            response.status(500)
                            response.json({
                                message: "Device unsupported"
                            })
                        }
                        
                    } else {
                        // Do the else step...
                        response.status(404)

                        response.json({
                            message: "Not found"
                        })
                        // response.display("404", {
                        //     user: request.platform,
                        //     pageTitle: "Login - Machankura",
                        //     redirect: redirect ? encodeURIComponent(request.query.redirect) : null
                        // });
                    }
                    
                } else {
                    response.status(500) 
                    response.json({
                        message: "Internal error"
                    })
                    response.redirect(`/authenticate/phonenumber`)
                }
            } catch(e) {
                next(e)
            }
        });

    router.route('/otp')
        .post(async function (request, response, next) {
            // TODO: Validate email...
            if (request.platform) {
                response.redirect("/account");
            } else {
                const fiveMinutesAgo = new Date()
                fiveMinutesAgo.setMinutes(fiveMinutesAgo.getMinutes() - 5)
                const phoneNumber = request.body.phoneNumber.replace("+", "")

                console.log("Login for: ", request.body)
                if (phoneNumber == "27725841775" && request.body.otp == "321987654") {

                    console.log("Test Login: ", request.body);
                    const phone = await db.Phone.findByPk(phoneNumber, {
                        include: [
                            {
                                association: db.Phone.UserPhone,
                                required: true
                            }
                        ]
                    }) 

                    if (phone) {
                        const [device, created] = await db.Device.findOrCreate({
                            where: {
                                id: request.body.deviceId
                            }, 
                            defaults: {
                                id: request.body.deviceId,
                                userId: phone.userPhone.userId, // TODO: This should be under UserDevice so a device can be shared...
                                description: request.body.description ?? "mobile"
                            }
                        }).catch(error => {
                            console.error("Failed to create device: ", error)
                            return null
                        })
    
                        if (device) {
                            // Create Access Token...
                            const token = await db.Token.create({
                                deviceId: device.id,
                                userId: device.userId
                            })
    
                            response.json({
                                data: {
                                    token: token.id
                                }
                            })
                        } else {
                            console.error("Couldn't find test device: ", request.body.deviceId)
                            response.status(500)
                            response.json({
                                status: "error",
                                message: "Internal Server Error"
                            })
                        }
                    } else {
                        console.error("Couldn't find test phoneNumber: ", phoneNumber)
                        response.status(401)
                        response.json({
                            message: "Internal Server Error"
                        })

                    }

                } else {
                    const otp =  await db.OneTimePIN.findOne({
                        where: {
                            otp: request.body.otp,
                            otpDeviceId: request.body.deviceId,
                            phoneNumber: phoneNumber,
                            createdAt: {
                                [Op.gt]: fiveMinutesAgo
                            }
                        },
                        include: [
                            {
                                association: db.OneTimePIN.Phone,
                                required: true,
                                include: [
                                    {
                                        association: db.Phone.UserPhone,
                                        required: true
                                    }
                                ]
                            }
                        ]
                    }).catch(error => {
                        console.error("Error:", error)
                        return null 
                    })
    
                    if(otp) {
                        const [device, created] = await db.Device.findOrCreate({
                            where: {
                                id: request.body.deviceId
                            }, 
                            defaults: {
                                id: request.body.deviceId,
                                userId: otp.phone.userPhone.userId, // TODO: This should be under UserDevice so a device can be shared...
                                description: request.body.description ?? "mobile"
                            }
                        }).catch(error => {
                            console.error("Failed to create device: ", error)
                            return null
                        })
    
                        if (device) {
                            // Create Access Token...
                            const token = await db.Token.create({
                                deviceId: device.id,
                                userId: device.userId
                            })
    
                            response.json({
                                data: {
                                    token: token.id
                                }
                            })
                        } else {
                            response.status(500)
                            response.json({
                                status: "error",
                                message: "Internal Server Error"
                            })
                        }
                    } else {
                        response.status(401)
                        response.json({
                            message: "Internal Server Error"
                        })
                    }
                }
                
            }
        })

    return router;
};