/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const exchangeRate = require("machankura.exchange-rate")

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.platform) {

                const countries = await db.Country.findAll({
                    include: [
                        {
                            association: db.Country.ServiceCountry,
                            where: {
                                status: "GENERALLY_AVAILABLE"
                            },
                            required: true
                        }
                    ],
                    order: [
                        ["currencyTicker"]
                    ]
                })
                const currencies = await exchangeRate.getMultipleBitcoinExchangeRates(
                    countries.concat({
                        name: "United States of America",
                        currencyTicker: "USD",
                        // TODO: Flag...
                    })
                );

                const currenciesWithSatsAndBitcoin = [
                    {
                        currency: "sats",
                        marketExchangeRate: "100 000 000", 
                        marketValue: 100_000_000,
                    },
                    // {
                    //     currency: "BTC",
                    //     marketExchangeRate: "1", 
                    //     marketValue: 1,
                    // },
                ].concat(
                    currencies
                )
                response.json({
                    status: "ok",
                    data: currenciesWithSatsAndBitcoin.map(c => {
                        return {
                            currencyTicker: c.currency,
                            currencyExchangeRate: c.marketValue
                        }
                    })
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    return router;
};