const config = require("config")
const express = require("express")
const path = require("path")

const db = require('machankura.db');

module.exports = function (options) {
    // var express = options.express;
    
    const router = express.Router();

    router.use((request, response, next) => {

        console.log("v1 Path: ", request.originalUrl)
        next()
    });

    const authenticateRouter = require('./authenticate/')();

    router.use('/authenticate', authenticateRouter);

    // Load platform being serviced
    router.use(async (request, response, next) => {
        const apiKeyHeader = request.headers["authorization"].split(" ").at(-1);

        const apiKey = await db.ApiKey.findByPk(apiKeyHeader, {
            include: [
                {
                    association: db.ApiKey.Platform
                }
            ]
        }).catch(error => {
            console.log("Error finding API key: ", error)
            return null
        });

        if (apiKey) {
            request.platform = apiKey.platform
    
            next()
        } else {
            response.status(401)
            response.json({
                status: "error",
                message: "Unauthorized"
            })
        }
    })

    router.route('/favicon.ico')
        .get(function(request, response, next)  {
            response.sendFile(
                path.resolve("server/static/img/icon/favicon.ico")
            )
        });

    const platformRouter = require('./platform/')();
    const walletsRouter = require('./wallets/')();
    const currenciesRouter = require('./currencies/')();
    const usernamesRouter = require('./usernames/')();
    const usersRouter = require('./users/')();
    // const orangePeelingRouter = require('./orange-peeling/')();

    router.use('/currencies', currenciesRouter);
    router.use('/wallets', walletsRouter);
    router.use('/platform', platformRouter);
    router.use('/usernames', usernamesRouter);
    router.use('/users', usersRouter);
    // router.use('/orange-peeling', orangePeelingRouter);
        
    // 404 pages...
    router.use((request, response, next) => {

        if(!response.headersSent) {
            response.status(404);

            response.json({
                message: "Resource Not Found"
            })
            
            // response.type('txt').send('Resource Not found');
        }
    });

    // 500 pages... 
    router.use((error, request, response, next) => {
        
        if(error && !response.headersSent) {
            console.error("We have an error: ", error)
            response.status(500);

            // TODO: Check if production and then show stack...
            response.json({
                message: "Internal Server Error"
            })
            
            
            // response.type('txt').send('Internal server error');
        }
    });

    return router;
};