/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.platform) {
                const clans = await db.Wallet.findAll({
                    include: [
                        {
                            association: db.Wallet.Name
                        },
                        {
                            association: db.Wallet.WalletMembers,
                            required: true,
                            where: {
                                userId: request.platform.id
                            }
                        }
                    ],
                    order: [
                        ['createdAt', 'DESC'],
                    ],
                    limit: 21,
                    // offset: 21
                })


                response.json({
                    data: clans
                })
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/dashboard')
        .get(async function (request, response, next) {
            if (request.platform) {
                const clans = await db.Wallet.findAll({
                    include: [
                        {
                            association: db.Wallet.Name
                        },
                        {
                            association: db.Wallet.WalletMembers,
                            required: true,
                            where: {
                                userId: request.platform.id
                            }
                        }
                    ],
                    order: [
                        ['createdAt', 'DESC'],
                    ],
                    limit: 21,
                    // offset: 21
                })

                // TODO: Count invitations...
                const pendingInvitationCount = await db.WalletMemberInvitation.count({
                    where: {
                        userId: request.platform.id,
                        status: "pending"
                    }
                })

                response.json({
                    data: {
                        pendingInvitationCount,
                        clans
                    }
                })
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/u/create') 
        .post(async function (request, response, next) {
            if (request.platform) {
                const existingName = await db.Name.findOne({
                    where: {
                        text: request.body.name
                    }
                }).catch(error => {
                    return null
                })

                if (existingName) {
                    response.status(409)
                    return response.json({
                        status: "error",
                        message: "Username is already in use"
                    })
                } else {
                    const actionPolicy = await db.ActionPolicy.findByPk(
                        request.body.actionPolicyId
                    );

                    if (actionPolicy) {
                        const t = await db.sequelize.transaction();

                        const name = await db.Name.create({
                            text: request.body.name
                        }, {
                            transaction: t
                        }).catch(error => {
                            console.error("Name Error: ", error) 
                            return null
                        })

                        if (name) {
                            const WalletMembers = [
                                {
                                    userId: request.platform.id,
                                    type: "admin"
                                }
                            ]
                            const clan = await db.Wallet.create({
                                nameId: name.id,
                                creatorId: request.platform.id,
                                clanRoot: {
                                    id: `c:${name.id}`,
                                    userName: {
                                        nameId: name.id
                                    }
                                },
                                walletName: {
                                    nameId: name.id
                                }, 
                                WalletMembers,
                                walletActionPolicy: {
                                    id: `c:${name.id}`,
                                    actionPolicyId: actionPolicy.id,
                                    n: db.WalletActionPolicy.calculateN(
                                        actionPolicy.id,
                                        WalletMembers
                                    )
                                }
                                // TODO: create clan action policy
                            }, {
                                include: [
                                    // {
                                    //     association: db.Wallet.ClanRoot,
                                    //     include: [
                                    //         {
                                    //             association: db.User.UserName,
                                    //             include: [
                                    //                 {
                                    //                     association: db.UserName.Name
                                    //                 }
                                    //             ]
                                    //         }
                                    //     ]
                                    // },
                                    {
                                        association: db.Wallet.WalletName,
                                        include: [
                                            {
                                                association: db.WalletName.Name
                                            }
                                        ]
                                    },
                                    {
                                        association: db.Wallet.WalletMembers
                                    },
                                    {
                                        association: db.Wallet.WalletActionPolicy
                                    }
                                ],
                                transaction: t
                            }).catch(error => {
                                console.error("Error creating clan: ", error)
                                return null
                            })

                            if (clan) {
                                await t.commit()
                                return response.json({
                                    data: clan.id
                                })
                            } else {
                                await t.rollback();
                                response.status(500)
                                response.json({
                                    status: "error",
                                    message: "Server error creating clan"
                                })
                            }
                        } else {
                            await t.rollback();
                            response.status(500)
                            response.json({
                                status: "error",
                                message: "Server error booking clan"
                            })
                        }

                        
                    } else {
                        response.status(428)
                        response.json({
                            status: "error",
                            message: "Action policy is required"
                        })
                    }
                }
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    const userRouter = require('./user')();

    router.use('/:userId', userRouter);

    return router;
};