/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const exchangeRate = require("machankura.exchange-rate")
const queries = require("machankura.lib/queries")
const { Op, Association } = require("sequelize")

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.user) {
                if (request.user.userWallet) {
                    response.json({
                        status: "ok",
                        data: request.user.userWallet.wallet
                    })
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "Uninitated wallet"
                    })
                }
                
            } else {
                response.status(404)
                response.json({
                    status: "error",
                    message: "User not found"
                })
            }
        })

    router.route('/initiate')
        .post(async function (request, response, next) {
            if (request.user) {

                if (request.user.userWallet) {
                    response.json({
                        status: "ok",
                        data: request.user.userWallet.wallet
                    })
                } else {
                    const unitOfAccountTicker = request.body.unitOfAccount ?? "sats"
                    // TODO: Create wallet
                    const wallet = await db.Wallet.create({
                        creatorId: request.user.id,
                        unitOfAccountTicker, // IF we don't have the unitOfAccount then this will fail...
                        walletName: {
                            name: {
                                text: request.body.walletName
                            }
                        },
                        userWallet: {
                            userId: request.user.id
                        }
                    }, {
                        include: [
                            {
                                association: db.Wallet.UserWallet
                            },
                            {
                                association: db.Wallet.WalletName,
                                include: [
                                    {
                                        association: db.WalletName.Name
                                    }
                                ]
                            }
                        ]
                    }).catch(error => {
                        console.error("Failed to create wallet: ", error)
                        return null
                    })
                    if (wallet) {
                        response.json({
                            status: "ok",
                            data: wallet
                        })
                    } else {
                        response.status(500)
                        response.json({
                            status: "error",
                            message: "Failed to initiate wallet"
                        })
                    }
                    
                }
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/unit-of-account-balance')
        .get(async function (request, response, next) {
            if (request.wallet) {

                const satsBalance = await request.wallet.getWalletBallance(db)
                const btcBalance = Number(satsBalance/100_000_000).toFixed(8)

                const country = await getCountry(db, request.wallet) 
                // queries.country.getServicableCountryByPhoneNumber(
                //     db, 
                //     request.wallet.userPhones?.at(0)?.phoneNumber
                // )
                const bitcoinExchangeRate = await exchangeRate.getBitcoinExchangeRate(country, "USD")
                const fiatBalance = Math.round(btcBalance*bitcoinExchangeRate.marketValue)

                const count = await db.Transfer.count({
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    }
                })

                const clanMetaData = await getClanMetaData(
                    db,
                    request.wallet
                )

                response.json({
                    status: "ok",
                    data: {
                        user: request.wallet,
                        satsBalance,
                        btcBalance,
                        fiatBalance,
                        fiatCurrency: bitcoinExchangeRate.currency,
                        fiatMarketValue: bitcoinExchangeRate.marketValue,
                        preferredCurrency: {
                            currencyTicker: bitcoinExchangeRate.currency,
                            currencyExchangeRate: bitcoinExchangeRate.marketValue,
                        },
                        // TODO: Add fiatBalance and faitCurrency... with 
                        transferCount: count,
                        pendingActionProposalCount: clanMetaData?.pendingActionProposalCount
                    }
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/satoshis-balance')
        .get(async function (request, response, next) {
            if (request.user) {

                const satsBalance = await request.wallet.getWalletBallance(db)
                const btcBalance = Number(satsBalance/100_000_000).toFixed(8)

                const country = await getCountry(db, request.wallet) 
                // queries.country.getServicableCountryByPhoneNumber(
                //     db, 
                //     request.wallet.userPhones?.at(0)?.phoneNumber
                // )
                const bitcoinExchangeRate = await exchangeRate.getBitcoinExchangeRate(country, "USD")
                const fiatBalance = Math.round(btcBalance*bitcoinExchangeRate.marketValue)

                const count = await db.Transfer.count({
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    }
                })

                const clanMetaData = await getClanMetaData(
                    db,
                    request.wallet
                )

                response.json({
                    status: "ok",
                    data: {
                        user: request.wallet,
                        satsBalance,
                        btcBalance,
                        fiatBalance,
                        fiatCurrency: bitcoinExchangeRate.currency,
                        fiatMarketValue: bitcoinExchangeRate.marketValue,
                        preferredCurrency: {
                            currencyTicker: bitcoinExchangeRate.currency,
                            currencyExchangeRate: bitcoinExchangeRate.marketValue,
                        },
                        // TODO: Add fiatBalance and faitCurrency... with 
                        transferCount: count,
                        pendingActionProposalCount: clanMetaData?.pendingActionProposalCount
                    }
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })
        
    // TODO: Might want to load up the ../wallets/wallet with the request.wallet = request.user.userWallet.wallet
    return router;
};