/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const exchangeRate = require("machankura.exchange-rate")
const queries = require("machankura.lib/queries")
const { Op } = require("sequelize")

const getCountry = async function (db, user) {
    if (user.userPhones?.at(0)?.phoneNumber) {
        return await queries.country.getServicableCountryByPhoneNumber(
            db, 
            user.userPhones?.at(0)?.phoneNumber
        )
    } else {
        return null
    }
}

const getClanMetaData = async (db, wallet) => {
    if (wallet.id.startsWith(`w:${request.platform.name}:`)) {
        const pendingActionProposalCount = await db.ActionProposal.count({
            where: {
                [Op.and]: [
                    { clanRootId: wallet.id },
                    { 
                        proposerId: {
                            [Op.ne]: wallet.clanRoot.WalletMembers[0].id,
                        } 
                    },
                    db.sequelize.where(db.Sequelize.literal(db.normalize(`(SELECT coalesce(COUNT("actionProposalReviews"."id"), 0) FROM "actionProposalReviews" WHERE "actionProposalReviews"."WalletMemberId" = '${wallet.clanRoot.WalletMembers[0].id}' AND "actionProposalReviews"."actionProposalId" = "actionProposal"."id")`)), Op.eq, 0),
                ]
            }
        }).catch(error => {
            console.error("Proposal count error: ", error)
            return 0
        })

        return {
            pendingActionProposalCount
        }
    } else {
        return null
    }
}

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.wallet) {

                const satsBalance = await request.wallet.getWalletBallance(db)
                const btcBalance = Number(satsBalance/100_000_000).toFixed(8)

                const country = await getCountry(db, request.wallet) 
                // queries.country.getServicableCountryByPhoneNumber(
                //     db, 
                //     request.wallet.userPhones?.at(0)?.phoneNumber
                // )
                const bitcoinExchangeRate = await exchangeRate.getBitcoinExchangeRate(country, "USD")
                const fiatBalance = Math.round(btcBalance*bitcoinExchangeRate.marketValue)

                const count = await db.Transfer.count({
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    }
                })

                const clanMetaData = await getClanMetaData(
                    db,
                    request.wallet
                )

                response.json({
                    status: "ok",
                    data: {
                        user: request.wallet,
                        satsBalance,
                        btcBalance,
                        fiatBalance,
                        fiatCurrency: bitcoinExchangeRate.currency,
                        fiatMarketValue: bitcoinExchangeRate.marketValue,
                        preferredCurrency: {
                            currencyTicker: bitcoinExchangeRate.currency,
                            currencyExchangeRate: bitcoinExchangeRate.marketValue,
                        },
                        // TODO: Add fiatBalance and faitCurrency... with 
                        transferCount: count,
                        pendingActionProposalCount: clanMetaData?.pendingActionProposalCount
                    }
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/bitcoin-balance')
        .get(async function (request, response, next) {
            if (request.wallet) {

                const satsBalance = await request.wallet.getWalletBallance(db)
                const btcBalance = Number(satsBalance/100_000_000).toFixed(8)

                const country = await getCountry(db, request.wallet) 
                // queries.country.getServicableCountryByPhoneNumber(
                //     db, 
                //     request.wallet.userPhones?.at(0)?.phoneNumber
                // )
                const bitcoinExchangeRate = await exchangeRate.getBitcoinExchangeRate(country, "USD")
                const fiatBalance = Math.round(btcBalance*bitcoinExchangeRate.marketValue)

                const count = await db.Transfer.count({
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    }
                })

                const clanMetaData = await getClanMetaData(
                    db,
                    request.wallet
                )

                response.json({
                    status: "ok",
                    data: {
                        user: request.wallet,
                        satsBalance,
                        btcBalance,
                        fiatBalance,
                        fiatCurrency: bitcoinExchangeRate.currency,
                        fiatMarketValue: bitcoinExchangeRate.marketValue,
                        preferredCurrency: {
                            currencyTicker: bitcoinExchangeRate.currency,
                            currencyExchangeRate: bitcoinExchangeRate.marketValue,
                        },
                        // TODO: Add fiatBalance and faitCurrency... with 
                        transferCount: count,
                        pendingActionProposalCount: clanMetaData?.pendingActionProposalCount
                    }
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/unit-of-account-balance')
        .get(async function (request, response, next) {
            if (request.wallet) {

                const satsBalance = await request.wallet.getWalletBallance(db)
                const btcBalance = Number(satsBalance/100_000_000).toFixed(8)

                const country = await getCountry(db, request.wallet) 
                // queries.country.getServicableCountryByPhoneNumber(
                //     db, 
                //     request.wallet.userPhones?.at(0)?.phoneNumber
                // )
                const bitcoinExchangeRate = await exchangeRate.getBitcoinExchangeRate(country, "USD")
                const fiatBalance = Math.round(btcBalance*bitcoinExchangeRate.marketValue)

                const count = await db.Transfer.count({
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    }
                })

                const clanMetaData = await getClanMetaData(
                    db,
                    request.wallet
                )

                response.json({
                    status: "ok",
                    data: {
                        user: request.wallet,
                        satsBalance,
                        btcBalance,
                        fiatBalance,
                        fiatCurrency: bitcoinExchangeRate.currency,
                        fiatMarketValue: bitcoinExchangeRate.marketValue,
                        preferredCurrency: {
                            currencyTicker: bitcoinExchangeRate.currency,
                            currencyExchangeRate: bitcoinExchangeRate.marketValue,
                        },
                        // TODO: Add fiatBalance and faitCurrency... with 
                        transferCount: count,
                        pendingActionProposalCount: clanMetaData?.pendingActionProposalCount
                    }
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })
        
    return router;
};