/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')

module.exports = function () {
    var router = express.Router();

    router.use(async (request, response, next) => {
        if (request.platform) {
            console.log("Base URL: ", request.baseUrl)
            console.log("Original URL: ", request.originalUrl)

            request.originalUrl = request.originalUrl.at(-1) == "/" ? request.originalUrl.slice(0,-1) : request.originalUrl
            const clanId = request.baseUrl.split("/").at(-1);

            console.log("ClanID: ", clanId)
            // TODO: Find user...
            const wallet = await db.Wallet.findOne({
                include: [
                    {
                        association: db.User.ClanRoot,
                        required: true,
                        where: {
                            id: clanId
                        },
                        include: [
                            {
                                association: db.Wallet.WalletMembers,
                                required: true,
                                where: {
                                    userId: request.platform.id
                                }
                            },
                            {
                                association: db.Wallet.ClanActionPolicy
                            }
                        ]
                    },
                    {
                        association: db.User.UserRoles,
                        include: {
                            model: db.Role,
                            attributes: ['type'],
                        }
                    },
                    {
                        association: db.User.UserName,
                        include: [
                            {
                                association: db.UserName.Name
                            }
                        ]
                    },
                    {
                        association: db.User.UserPhones,
                        include: [
                            {
                                association: db.UserPhone.Phone
                            }
                        ]
                    }
                ]
            })
            if (wallet) {
                request.wallet = wallet

                next()
                // if (user.clanRoot.clanActionPolicy || request.path == "/settings/policy") {
                //     next();
                // } else {
                //     response.redirect(`/clans/${user.clanRoot.id}/settings/policy`);
                // }
            } else {
                response.status(404)
                response.json({
                    status: "error",
                    message: "Clan not found"
                })
            }
        } else {
            response.status(401)
            response.json({
                status: "error",
                message: "Unauthorized"
            })
        }
    })

    // router.route('/')
    //     .get(async function (request, response, next) {
    //         if (request.platform) {
    //             response.json({
    //                 data: request.platform
    //             });
    //         } else {
    //             response.redirect('/account/authenticate')
    //         }
    //     })

    const dashboardRouter = require('./dashboard')();
    const transfersRouter = require('./transfers')();
    const sendRouter = require('./send')();

    const membersRouter = require('./members')();
    const settingsRouter = require('./settings')();

    router.use('/dashboard', dashboardRouter);
    router.use('/transfers', transfersRouter);
    // router.use('/send', sendRouter);

    router.use('/members', membersRouter);
    router.use('/settings', settingsRouter);

    return router;
};