/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
var queries = require("machankura.lib/queries");
const { Op } = require("sequelize")

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const pageSize = Number(request.query.pageSize) || 15
                const page = Number(request.query.page) || 0

                const { count, rows } = await db.WalletMember.findAndCountAll({
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    attributes: {
                        include: [
                            [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = "WalletMember"."userId" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL)`)), 'deposited'],
                            [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "WalletMemberDeposits" LEFT JOIN "transfers" ON "WalletMemberDeposits"."transferId" = "transfers"."id" WHERE "WalletMemberDeposits"."WalletMemberId" = "WalletMember"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'WalletMemberTotalDeposited'],
                            [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = '${request.wallet.id}' AND "transfers"."destinationId" = "WalletMember"."userId" AND "transfers"."deletedAt" IS NULL)`)), 'withdrew']
                        ]
                    },
                    limit: pageSize,
                    offset: page*pageSize,
                    order: [
                        ['createdAt', 'DESC'],
                    ],
                    include: [
                        {
                            association: db.WalletMember.User,
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                    // Add pagination...
                }).catch(error => {
                    console.error("Error getting members: ", error)
                    return {
                        count: 0,
                        rows: []
                    }
                })
                response.json({
                    data: {
                        items: rows,
                        itemCount: count,
                        page,
                        totalPages: Math.ceil(count/pageSize)
                    }
                })                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/extended')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const pageSize = Number(request.query.pageSize) || 15
                const page = Number(request.query.page) || 0

                const { count, rows } = await db.WalletMember.findAndCountAll({
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    attributes: {
                        include: [
                            [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = "WalletMember"."userId" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL)`)), 'deposited'],
                            [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "WalletMemberDeposits" LEFT JOIN "transfers" ON "WalletMemberDeposits"."transferId" = "transfers"."id" WHERE "WalletMemberDeposits"."WalletMemberId" = "WalletMember"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'WalletMemberTotalDeposited'],
                            [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = '${request.wallet.id}' AND "transfers"."destinationId" = "WalletMember"."userId" AND "transfers"."deletedAt" IS NULL)`)), 'withdrew']
                        ]
                    },
                    limit: pageSize,
                    offset: page*pageSize,
                    order: [
                        ['createdAt', 'DESC'],
                    ],
                    include: [
                        {
                            association: db.WalletMember.User,
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                    // Add pagination...
                }).catch(error => {
                    console.error("Error getting members: ", error)
                    return {
                        count: 0,
                        rows: []
                    }
                })

                const pendingMemberInviteCount = await db.WalletMemberInvitation.count({
                    where: {
                        clanId: request.wallet.clanRoot.id,
                        status: "pending"
                    },
                }).catch(error => {
                    console.error("Failed to get pending member count: ", error)
                    return 0
                });

                const declinedMemberInviteCount = await db.WalletMemberInvitation.count({
                    where: {
                        clanId: request.wallet.clanRoot.id,
                        status: "declined"
                    },
                }).catch(error => {
                    console.error("Failed to get declined member count: ", error)
                    return 0
                });

                response.json({
                    data: {
                        paginatedWalletMemberList: {
                            items: rows,
                            itemCount: count,
                            page,
                            totalPages: Math.ceil(count/pageSize)
                        },
                        pendingMemberInviteCount,
                        declinedMemberInviteCount,
                    }
                })                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/invite')
        .post(async function (request, response, next) {
            if (request.wallet) {
                const user = await queries.user.getUser(
                    db,
                    request.body.username,
                    [
                        {
                            association: db.User.WalletMembers,
                            required: false,
                            where: {
                                clanId: request.wallet.clanRoot.id,
                            }
                        },
                        {
                            association: db.User.WalletMemberInvitations,
                            required: false,
                            where: {
                                clanId: request.wallet.clanRoot.id,
                                status: "pending"
                            }
                        }
                    ]
                    
                )
                if (user) {
                    if (user.WalletMembers.length > 0) {
                        response.status(400)
                        response.json({
                            status: "error",
                            message: `User ${request.body.username} is already a member of the clan`
                        })
                    } else if (user.WalletMemberInvitations.length > 0) {
                        response.status(400)
                        response.json({
                            status: "error",
                            message: `Invite to ${request.body.username} is still pending`
                        })
                    } else {
                        // Create invitation...
                        const WalletMemberInvitation = await db.WalletMemberInvitation.create({
                            clanId: request.wallet.clanRoot.id,
                            userId: user.id,
                            type: request.body.type,
                            status: "pending"
                        }).catch(error => {
                            console.error("Error: ", error);
                            return null
                        })

                        if (WalletMemberInvitation) {
                            response.json({
                                data: WalletMemberInvitation.id
                            })
                        } else {
                            response.status(400)
                            response.json({
                                status: "error",
                                message: `Failed to invite ${request.body.username} to clan.`
                            })
                        }
                    }
                } else {
                    response.status(400)
                    response.json({
                        status: "error",
                        message: `User with username ${request.body.username} not found`
                    })
                }               
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/invite/pending')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const pageSize = Number(request.query.pageSize) || 15
                const page = Number(request.query.page) || 0

                const { count, rows } = await db.WalletMemberInvitation.findAndCountAll({
                    where: {
                        clanId: request.wallet.clanRoot.id,
                        status: "pending"
                    },
                    limit: pageSize,
                    offset: page*pageSize,
                    order: [
                        ['createdAt', 'DESC'],
                    ],
                    include: [
                        {
                            association: db.WalletMemberInvitation.User,
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                    // Add pagination...
                })
                response.json({
                    data: {
                        items: rows,
                        itemCount: count,
                        page,
                        totalPages: Math.ceil(count/pageSize)
                    }
                })
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

        router.route('/invite/declined')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const pageSize = Number(request.query.pageSize) || 15
                const page = Number(request.query.page) || 0

                const { count, rows } = await db.WalletMemberInvitation.findAndCountAll({
                    where: {
                        clanId: request.wallet.clanRoot.id,
                        status: "declined"
                    },
                    limit: pageSize,
                    offset: page*pageSize,
                    order: [
                        ['createdAt', 'DESC'],
                    ],
                    include: [
                        {
                            association: db.WalletMemberInvitation.User,
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                    // Add pagination...
                })
                response.json({
                    data: {
                        items: rows,
                        itemCount: count,
                        page,
                        totalPages: Math.ceil(count/pageSize)
                    }
                    
                })
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/:WalletMemberId')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const WalletMember = await db.WalletMember.findByPk(request.params.WalletMemberId, {
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    include: [
                        {
                            association: db.WalletMember.User,
                            attributes: { 
                                include: [
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."destinationId" = "user"."id" AND "transfers"."sourceId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid') )`)), 'clanTotalWithdrawn'],
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = "user"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'clanTotalDeposited'],
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "WalletMemberDeposits" LEFT JOIN "transfers" ON "WalletMemberDeposits"."transferId" = "transfers"."id" WHERE "WalletMemberDeposits"."WalletMemberId" = "WalletMember"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'WalletMemberTotalDeposited']
                                ]
                            },
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                }).catch(error => {
                    console.error("Error: ", error)
                    return null
                })
                
                if (WalletMember) {
                    response.json({
                        data: WalletMember
                    })
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "Clan member not found"
                    })
                }
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })


    router.route('/:WalletMemberId/remove')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const WalletMember = await db.WalletMember.findByPk(request.params.WalletMemberId, {
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    include: [
                        {
                            association: db.WalletMember.User,
                            attributes: { 
                                include: [
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."destinationId" = "user"."id" AND "transfers"."sourceId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid') )`)), 'clanTotalWithdrawn'],
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = "user"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'clanTotalDeposited'],
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "WalletMemberDeposits" LEFT JOIN "transfers" ON "WalletMemberDeposits"."transferId" = "transfers"."id" WHERE "WalletMemberDeposits"."WalletMemberId" = "WalletMember"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'WalletMemberTotalDeposited']
                                ]
                            },
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                }).catch(error => {
                    console.error("Error: ", error)
                    return null
                })
                
                if (WalletMember) {
                    response.json({
                        data: WalletMember
                    })
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "Clan member not found"
                    })
                }
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })
        .post(async function (request, response, next) {
            if (request.wallet) {
                const WalletMember = await db.WalletMember.findByPk(request.params.WalletMemberId, {
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    include: [
                        {
                            association: db.WalletMember.User,
                            attributes: { 
                                include: [
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."destinationId" = "user"."id" AND "transfers"."sourceId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid') )`)), 'clanTotalWithdrawn'],
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = "user"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'clanTotalDeposited'],
                                    [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "WalletMemberDeposits" LEFT JOIN "transfers" ON "WalletMemberDeposits"."transferId" = "transfers"."id" WHERE "WalletMemberDeposits"."WalletMemberId" = "WalletMember"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'WalletMemberTotalDeposited'],
                                ]
                            },
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                }).catch(error => {
                    console.error("Error: ", error)
                    return null
                })
                
                if (WalletMember) {
                    // Create a remove member proposal...
                    const proposedWalletMember = WalletMember.toJSON();
                    const removeMemberProposalId = `${request.wallet.clanRoot.WalletMembers[0].id}:${WalletMember.id}:r`

                    proposedWalletMember.deletedAt = new Date();
                    proposedWalletMember.updatedAt = new Date();

                    const actionProposal = await db.ActionProposal.create({
                        id: removeMemberProposalId,
                        proposerId: request.wallet.clanRoot.WalletMembers[0].id,
                        clanRootId: request.wallet.id,
                        clanActionPolicyId: request.wallet.clanRoot.clanActionPolicy.id,
                        actionSideEffect: {
                            id:  removeMemberProposalId,
                            clanRootId: request.wallet.id,
                            queryData: proposedWalletMember,
                            actionMethod: "update",
                            actionModel: "WalletMember"
                        }
                    }, {
                        include: [
                            {
                                association: db.ActionProposal.ActionSideEffect
                            }
                        ]
                    }).catch(error => {
                        console.error("Failed to propose removal: ", error)
                        return null
                    })

                    if (actionProposal) {
                        response.json({
                            data: actionProposal
                        })
                    } else {
                        response.status(400)
                        response.json({
                            status: "error",
                            message: "Failed to remove member"
                        })
                    }
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "Couldn't find member"
                    })
                }
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/:WalletMemberId/edit')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const WalletMember = await db.WalletMember.findByPk(request.params.WalletMemberId, {
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    include: [
                        {
                            association: db.WalletMember.User,
                            attributes: { 
                                // include: [
                                //     [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."destinationId" = "user"."id" AND "transfers"."sourceId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid') )`)), 'clanTotalWithdrawn'],
                                //     [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "transfers" WHERE "transfers"."sourceId" = "user"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'clanTotalDeposited'],
                                //     [db.Sequelize.literal(db.normalize(`(SELECT coalesce(SUM("transfers"."amountInSatoshis"), 0) FROM "WalletMemberDeposits" LEFT JOIN "transfers" ON "WalletMemberDeposits"."transferId" = "transfers"."id" WHERE "WalletMemberDeposits"."WalletMemberId" = "WalletMember"."id" AND "transfers"."destinationId" = '${request.wallet.id}' AND "transfers"."deletedAt" IS NULL AND ("transfers"."status" = 'settled' OR "transfers"."status" = 'paid'))`)), 'WalletMemberTotalDeposited']
                                // ]
                            },
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                }).catch(error => {
                    console.error("Error: ", error)
                    return null
                })
                
                if (WalletMember) {
                    response.json({
                        data: WalletMember
                    })
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "Couldn't find clan member"
                    })
                }
                
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })
        .post(async function (request, response, next) {
            if (request.wallet) {
                const WalletMember = await db.WalletMember.findByPk(request.params.WalletMemberId, {
                    where: {
                        clanId: request.wallet.clanRoot.id,
                    },
                    include: [
                        {
                            association: db.WalletMember.User,
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                {
                                    association: db.User.UserPhones,
                                    include: [
                                        {
                                            association: db.UserPhone.Phone
                                        }
                                    ]
                                },
                            ]
                        }
                    ]
                }).catch(error => {
                    console.error("Error: ", error)
                    return null
                })
                
                if (WalletMember) {
                    // Create a remove member proposal...
                    const proposedWalletMember = WalletMember.toJSON();
                    const editMembershipProposalId = `${request.wallet.clanRoot.WalletMembers[0].id}:${WalletMember.id}:${WalletMember.type}:${request.body.type}:u`

                    proposedWalletMember.type = request.body.type;

                    const actionProposal = await db.ActionProposal.create({
                        id: editMembershipProposalId,
                        proposerId: request.wallet.clanRoot.WalletMembers[0].id,
                        clanRootId: request.wallet.id,
                        clanActionPolicyId: request.wallet.clanRoot.clanActionPolicy.id,
                        actionSideEffect: {
                            id:  editMembershipProposalId,
                            clanRootId: request.wallet.id,
                            queryData: proposedWalletMember,
                            actionMethod: "update",
                            actionModel: "WalletMember"
                        }
                    }, {
                        include: [
                            {
                                association: db.ActionProposal.ActionSideEffect
                            }
                        ]
                    }).catch(error => {
                        console.error("Failed to propose removal: ", error)
                        return null
                    })

                    if (actionProposal) {
                        response.json({
                            data: actionProposal
                        })
                    } else {
                        response.status(400)
                        response.json({
                            status: "error",
                            message: "Failed to remove member"
                        })
                    }

                    
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "Clan member not found"
                    })
                }
                
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    return router;
};