/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const idGenerator = require("machankura.db/src/lib/id-generator")
const { Op } = require("sequelize")

const sparkoInvoice = require("machankura.lib/sparko/invoice")
const bolt11 = require("bolt11")
const lightningNodeUtil = require("machankura.lib/lightning-node-util")

module.exports = function () {
    var router = express.Router();

    router.route('/bolt11')
        .post(async function (request, response, next) {
            if (request.wallet) {
                // Send back bolt11 inv
                const amountInSatoshis = Number(request.body.amountInSatoshis)
                const label = idGenerator.generateUniqueId()

                const bolt11Invoice = await sparkoInvoice.invoice(
                    amountInSatoshis*1_000,
                    label,
                    request.body.description ?? "Send sats to Machankura user" ,
                    false,
                    request.body.expiry,
                )

                if (bolt11Invoice) {
                    // TODO: Save a pending transaction that expires in 15 minutes...
                    const bolt11Object = bolt11.decode(bolt11Invoice)
                    if (bolt11Object) {
                        const lightningNode = await lightningNodeUtil.machankuraLightningNode(
                            db,
                            bolt11Object
                        )
                        // TODO: Decode bolt11Invoice to get expiry...
                        if (lightningNode) {
                            const transfer = await db.Transfer.create({
                                status: "pending",
                                destinationId: request.wallet.id,
                                amountInSatoshis: amountInSatoshis,
                                feeInSatoshis: Math.ceil(amountInSatoshis*1/100), // TODO: Make this the receive fee...
                                destinationParameter: request.body.destinationParameter,
                                bolt11Transfer: {
                                    id: label,
                                    invoice: bolt11Invoice,
                                    status: "pending",
                                    amountInSatoshis: amountInSatoshis,
                                    expiresAt: new Date(bolt11Object.timeExpireDateString), // TODO: Set this from the bolt11 invoice
                                    lightningNodeId: lightningNode.id
                                },
                                comment: request.query.comment
                            }, {
                                include: [
                                    {
                                        association: db.Transfer.Bolt11Transfer
                                    }
                                ]
                            }).catch(error => {
                                console.error("Failed to store transfer: ", error)
                                console.log("Bolt11 Object: ", bolt11Object)
                                return null
                            })

                            if (transfer) {

                                console.log("Transfer: ", transfer.toJSON())
                                response.json({
                                    status: "ok",
                                    message: "Here is the bolt11invoice to receive sats",
                                    data: transfer.bolt11Transfer
                                })
                            } else {
                                response.status(500)
                                response.json({
                                    "status": "error",
                                    "reason": "Internal node failure" // Make an invoice from our lightning node
                                })
                            }
                        } else {
                            console.error("Couldn't find or create a lightning node")
                            response.status(500)
                            response.json({
                                "status": "error",
                                "reason": "Internal node failure" // Make an invoice from our lightning node
                            })
                        }
                    } else {
                        console.error("Couldn't decode the lightning invoice: ", bolt11Invoice)
                        response.status(500)
                        response.json({
                            "status": "error",
                            "reason": "Failed to initiate invoice" // Make an invoice from our lightning node
                        })
                    }
                } else {
                    console.error("Couldn't make invoice for ", username)
                    response.status(500)
                    response.json({
                        "status": "error",
                        "reason": "Failed to initiate payment" // Make an invoice from our lightning node
                    })
                }
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    return router;
};