/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const machankuraUserQueries = require("machankura.lib/queries/user")
const machankuraSendBitcoinHandler = require("../../../../handler/send")


module.exports = function () {
    var router = express.Router();

    router.route('/bolt11')
        .post(async function (request, response, next) {
            machankuraSendBitcoinHandler.sendBitcoinToBolt11Invoice(
                request,
                response,
                next
            )
        })

    router.route('/onchain')
        .post(async function (request, response, next) {
            machankuraSendBitcoinHandler.sendBitcoinToOnchain(
                request,
                response,
                next
            )
        })

    router.route('/username')
        .post(async function (request, response, next) {
            machankuraSendBitcoinHandler.sendBitcoinToMachankuraUser(
                request,
                response,
                next
            )
        })

    router.route('/phone-number')
        .post(async function (request, response, next) {
            if (request.wallet) {
                const destinationParameter = request.body.destinationParameter
        
                const receiverUser = machankuraUserQueries.getUser(
                    db,
                    destinationParameter
                )

                if (receiverUser) {
                    machankuraSendBitcoinHandler.sendBitcoinToMachankuraUser(
                        request,
                        response,
                        next,
                        receiverUser
                    )
                } else {
                    machankuraSendBitcoinHandler.giftBitcoinToPhoneNumber(
                        request,
                        response,
                        next
                    )
                }
            }
    
        })

    router.route('/lightning-address')
        .post(async function (request, response, next) {
            machankuraSendBitcoinHandler.sendBitcoinToLightningAddress(
                request,
                response,
                next
            )
        })

    return router;
};