/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
const db = require('machankura.db')

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const actionPolicies = await db.ActionPolicy.findAll()
                response.json({
                    data: actionPolicies
                })
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })
        .post(async function (request, response, next) {
            if (request.wallet) {
                if (request.wallet.clanRoot.clanActionPolicy) {
                    // Update policy
                    const actionPolicy = await db.ActionPolicy.findByPk(
                        request.body.actionPolicyId
                    );

                    if (actionPolicy) {

                        // TODO: Replace with WalletMemberCount
                        const WalletMembers = await db.WalletMember.findAll({
                            where: {
                                clanId: request.wallet.clanRoot.id
                            }
                        });

                        const proposedClanActionPolicy = request.wallet.clanRoot.clanActionPolicy;
                        proposedClanActionPolicy.actionPolicyId = actionPolicy.id;
                        proposedClanActionPolicy.n = db.ClanActionPolicy.calculateN(
                            actionPolicy.id,
                            WalletMembers
                        );

                        // await request.wallet.clanRoot.clanActionPolicy.save();

                        // Update  clan actionPolicy
                        const updateApprovalPoliciesId = `${request.wallet.clanRoot.WalletMembers[0].id}:${proposedClanActionPolicy.id}:${actionPolicy.id}:u`

                        const actionProposal = await db.ActionProposal.create({
                            id: updateApprovalPoliciesId,
                            proposerId: request.wallet.clanRoot.WalletMembers[0].id,
                            clanRootId: request.wallet.id,
                            clanActionPolicyId: request.wallet.clanRoot.clanActionPolicy.id,
                            actionSideEffect: {
                                id:  updateApprovalPoliciesId,
                                clanRootId: request.wallet.id,
                                queryData: proposedClanActionPolicy,
                                actionMethod: "update",
                                actionModel: "ClanActionPolicy"
                            }
                        }, {
                            include: [
                                {
                                    association: db.ActionProposal.ActionSideEffect
                                }
                            ]
                        }).catch(error => {
                            console.error("Failed to propose update: ", error)
                            return null
                        })

                        if (actionProposal) {
                            return response.json({
                                data: actionProposal
                            })
                        } else {
                            response.status(400)
                            response.json({
                                status: "error",
                                message: "Bad request"
                            })
                        }
                    } else {
                        // TODO: Tell user we couldn't find action policy
                        response.status(404)
                        response.json({
                            status: "error",
                            message: "Action Policy not found"
                        })
                    }
                } else {
                    const actionPolicies = await db.ActionPolicy.findAll()

                    const actionPolicy = await db.ActionPolicy.findByPk(
                        request.body.actionPolicyId
                    );

                    if (actionPolicy) {
                        const t = await db.sequelize.transaction();
                        const WalletMembers = await db.WalletMember.findAll({
                            where: {
                                clanId: request.wallet.clanRoot.id
                            }
                        });

                        const clanActionPolicy = await db.ClanActionPolicy.create({
                            id: request.wallet.id,
                            clanId: request.wallet.clanRoot.id,
                            actionPolicyId: actionPolicy.id,
                            n: db.ClanActionPolicy.calculateN(
                                actionPolicy.id,
                                WalletMembers
                            )
                        })

                        if (clanActionPolicy) {
                            response.json({
                                data: clanActionPolicy
                            })
                        } else {
                            response.status(400)
                            response.json({
                                status: "error",
                                message: "Bad request"
                            })
                        }
                    } else {
                        response.status(404)
                        response.json({
                            status: "error",
                            message: "Action policy not found"
                        })
                    }

                }
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    return router;
};