/**
 * This router handles things related to the web browser experience...
 */
const express = require('express')
var db = require('machankura.db')
const { Op } = require("sequelize")

module.exports = function () {
    var router = express.Router();

    router.route('/')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const transfers = await db.Transfer.findAll({
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    },
                    limit: 15,
                    order: [
                        ['createdAt', 'DESC'],
                    ]
                    // Add pagination...
                })

                response.json({
                    data: transfers
                })
                // response.display("transfers", {
                //     user: request.wallet,
                //     satsBalance: satsBalance,
                //     transfers: transfers,
                //     pageTitle: "Transfers - Machankura",
                // });
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/:transferId')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const transfer = await db.Transfer.findByPk(request.params.transferId, {
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    },
                    include: [
                        {
                            association: db.Transfer.Source,
                            include: [
                                {
                                    association: db.User.UserName,
                                    include: [
                                        {
                                            association: db.UserName.Name
                                        }
                                    ]
                                },
                                // {
                                //     association: db.User.UserPhones
                                // }
                            ],
                        },
                        {
                            association: db.Transfer.Bolt11Transfer,
                            // include: [
                            //     {
                            //         association: db.Bolt11Transfer.CryptoConvertWattsInvoice,
                            //         include: [
                            //             {
                            //                 association: db.CryptoConvertWattsInvoice.CryptoConvertWattsVoucher
                            //             }
                            //         ]
                            //     }
                            // ]
                        },
                        {
                            association: db.Transfer.BitcoinTransfer
                        },
                        {
                            association: db.Transfer.BitrefillInvoice
                        },
                        {
                            association: db.Transfer.AztecoVoucher
                        }
                    ],
                }).catch(error => {
                    console.error("Failed to find transfer: ", error)
                    return null
                })
                
                if (transfer) {
                    response.status(200)
                    response.json({
                        data: transfer
                    })
                } else {
                    response.status(404)
                    response.json({
                        message: "Transfer not found..."
                    })
                }
                
                
            } else {
                response.status(401)
                response.json({
                    status: "error",
                    message: "Unauthorized"
                })
            }
        })

    router.route('/:transferId/status')
        .get(async function (request, response, next) {
            if (request.wallet) {
                const transfer = await db.Transfer.findByPk(request.params.transferId, {
                    where: {
                        [Op.or]: {
                            sourceId: request.wallet.id,
                            destinationId: request.wallet.id
                        }
                    },
                }).catch(error => {
                    console.error("Failed to find transfer: ", error)
                    return null
                })
                
                if (transfer) {
                    response.status(200)
                    response.json({
                        data: transfer.status
                    })
                } else {
                    response.status(404)
                    response.json({
                        status: "error",
                        message: "not_found"
                    })
                }
                
                
            } else {
                next()
            }
        })

    return router;
};